/*
 * AbstractObject.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;

import org.jaxen.XPath;

import org.saxpath.SAXPathException;


/**
 *
 * @author  linuxwolf
 */
public abstract class AbstractObject implements StreamObject {
    //Constructors
    public AbstractObject() {
    }

    //Methods
    protected Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }
    
    public StreamContext getContext() {
        StreamElement   parent = getParent();
        StreamContext   ctx = null;
        
        if (parent != null) {
            ctx = (parent instanceof StreamContext) ? (StreamContext)parent : parent.getContext();
        }
        
        return ctx;
    }

    public NSI getNSI() {
        return null;
    }
    public String getLocalName() {
        return null;
    }
    public String getNamespacePrefix() {
        return null;
    }
    public String getNamespaceURI() {
        return null;
    }
    public String getQualifiedName() {
        return null;
    }
    
    public Object evaluateXPath(String expr) throws IllegalArgumentException {
        return evaluateXPath(expr, null);
    }
    public Object evaluateXPath(String expr, Map nss) throws IllegalArgumentException {
        XPath       xpath;
        Iterator    itr;
        Object      found = null;

        try {
            xpath = getDataFactory().createXPath(expr, nss);
            found = xpath.evaluate(this);
        } catch (SAXPathException saxpe) {
            IllegalArgumentException    e = new IllegalArgumentException(saxpe.getMessage());
            
            e.initCause(saxpe);
            throw e;
        }
        
        return found;
    }
    
    public StreamObject select(String expr) throws IllegalArgumentException {
        return select(expr, null);
    }
    public StreamObject select(String expr, Map nss) throws IllegalArgumentException {
        List    found = selectAll(expr, nss);
        
        return (StreamObject)((found.size() > 0) ? found.get(0) : null);
    }
    
    public Number selectNumber(String expr) throws IllegalArgumentException {
        return selectNumber(expr, null);
    }
    public Number selectNumber(String expr, Map nss) throws IllegalArgumentException {
        Object      found = evaluateXPath((expr != null) ? "number(" + expr + ")" : null, nss);
        
        return (Number)((found instanceof Number) ? found : null);
    }
    
    public String selectString(String expr) throws IllegalArgumentException {
        return selectString(expr, null);
    }
    public String selectString(String expr, Map nss) throws IllegalArgumentException {
        Object      found = evaluateXPath((expr != null) ? "string(" + expr + ")" : null, nss);
        
        return (String)((found instanceof String) ? found : null);
    }
    
    public List selectAll(String expr) throws IllegalArgumentException {
        return selectAll(expr, null);
    }
    public List selectAll(String expr, Map nss) throws IllegalArgumentException {
        Object      found = evaluateXPath(expr, nss);
        
        if (!(found instanceof List))
            throw new IllegalArgumentException("XPath does not evaluate to a node-set");
        return (List)found;
    }
    
    public Object clone() {
        return copy(null);
    }
    public StreamObject copy() {
        return copy(null);
    }
}