/*
 * BasicPacketBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;



import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class BasicPacketBuilder extends AbstractPacketBuilder {
    //"Constants"
    public static final NSI NAME_ERROR = new NSI("error", null);
    
    //Constructors
    public BasicPacketBuilder(BasicPacketNode packet) {
        super(packet);
    }
    
    //Methods
    protected Status processNestedElementStart(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI.Comparator      cmpr = NSI.NS_LOOSE_COMPARATOR;
        NSI                 name = s.getName();
        String              uri = name.getNamespaceURI();
        String              ln = name.getLocalName();
        StreamDataFactory   sdf = ctx.getDataFactory();
        StreamBuilder       b;
        Status              status = INVALID;


        if (NSI.NS_LOOSE_COMPARATOR.compare(name, NAME_ERROR) == 0) {
            setExtendedData(new PacketErrorNode(sdf).createBuilder());
            status = EXTENDED_DATA;
        } else {
            status = super.processNestedElementStart(s);
        }
        
        return status;
    }
    /** <p>
     * The the given <code>StreamNode</code> as extended data.</p>
     *
     *
     * @param node The extended data to add.
     * @throws IllegalStateException If the current state is not <code>{@link
     * #EXTENDED_DATA}</code>.
     */
    public void addExtendedData(StreamNode node) throws IllegalStateException {
        if (node instanceof PacketError) {
            getPacket().setError((PacketError)node);
        } else {
            super.addExtendedData(node);
        }
    }
}