/*
 * BasicPacketNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class BasicPacketNode extends AbstractPacket {
    //Constructors
    /** Creates a new instance of BasicPacketNode */
    public BasicPacketNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    public BasicPacketNode(StreamElement parent, BasicPacketNode base) {
        super(parent, base);
    }

    //Methods
    public PacketError getError() {
        Iterator    itr = listElements("error", getNamespaceURI()).iterator();
        
        return (PacketError)(itr.hasNext() ? itr.next() : null);
    }
    public void setError(PacketError err) {
        Iterator    itr = listElements("error", getNamespaceURI()).iterator();
        
        //Remove existing
        while (itr.hasNext()) {
            ((StreamElement)itr.next()).detach();
        }

        configurePacketError(err);
        super.add(err);
    }
    
    public StreamBuilder createBuilder() {
        return new BasicPacketBuilder(this);
    }
    
    public void add(StreamNode node) throws UnsupportedOperationException {
        if (node != null) {
            if (node instanceof PacketError) {
                configurePacketError((PacketError)node);
            }
            super.add(node);
        }
    }
    public void insert(int idx, StreamNode node) throws UnsupportedOperationException {
        if          (node == null) {
            return;
        } else if   (node instanceof StreamElement) {
            if (node instanceof PacketError) {
                configurePacketError((PacketError)node);
            }
            super.insert(idx, node);
        }
    }
    public void remove(StreamNode node) throws UnsupportedOperationException {
        if          (node == null) {
            return;
        } else if   (node instanceof StreamElement) {
            if (node instanceof PacketError) {
                configurePacketError(null);
            }
            super.remove(node);
        }
    }
    
    public StreamObject copy(StreamElement parent) {
        return new BasicPacketNode(parent, this);
    }
    
    public void checkNSI(NSI check) throws IllegalArgumentException {
        super.checkNSI(check);
        if (!Utilities.equateStrings(getLocalName(), check.getLocalName()))
            throw new IllegalArgumentException("local name must be equal");
    }
    public void checkElement(StreamElement check) throws IllegalArgumentException {
    }

    protected void configurePacketError(PacketError err) {
        if (err == null) {
            setType(null);
        } else {
            String      msg = err.normalizeTrimText();
            String      txt = err.getText();
            
            setType(ERROR);
            configureIETF(err);
            configureLegacy(err);
            
            if (!Utilities.isValidString(txt))
                err.setText(msg);
            if (Utilities.isValidString(msg))
                err.clearText();
        }
    }
    protected void configureLegacy(PacketError err) {
        PacketError.Type    type = err.getType();
        String              cond = err.getDefinedCondition();
        Number              codeObj = null;
        int                 codeVal = 0;
        
        try {
            codeObj = Integer.valueOf(err.getAttributeValue("code"));
        } catch (NumberFormatException nfe) {
        }
        
        if (codeObj == null) {
            if      (Utilities.equateStrings(cond, PacketError.BAD_REQUEST_CONDITION) && (type == PacketError.MODIFY))
                codeVal = 400;
            else if (Utilities.equateStrings(cond, PacketError.CONFLICT_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 409;
            else if (Utilities.equateStrings(cond, PacketError.FEATURE_NOT_IMPLEMENTED_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 501;
            else if (Utilities.equateStrings(cond, PacketError.FORBIDDEN_CONDITION) && (type == PacketError.AUTH))
                codeVal = 403;
            else if (Utilities.equateStrings(cond, PacketError.GONE_CONDITION) && (type == PacketError.MODIFY))
                codeVal = 302;
            else if (Utilities.equateStrings(cond, PacketError.INTERNAL_SERVER_ERROR_CONDITION) && (type == PacketError.WAIT))
                codeVal = 500;
            else if (Utilities.equateStrings(cond, PacketError.ITEM_NOT_FOUND_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 404;
            else if (Utilities.equateStrings(cond, PacketError.JID_MALFORMED_CONDITION) && (type == PacketError.MODIFY))
                codeVal = 400;
            else if (Utilities.equateStrings(cond, PacketError.NOT_ACCEPTABLE_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 406;
            else if (Utilities.equateStrings(cond, PacketError.NOT_ALLOWED_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 405;
            else if (Utilities.equateStrings(cond, PacketError.NOT_AUTHORIZED_CONDITION) && (type == PacketError.AUTH))
                codeVal = 401;
            else if (Utilities.equateStrings(cond, PacketError.PAYMENT_REQUIRED_CONDITION) && (type == PacketError.AUTH))
                codeVal = 402;
            else if (Utilities.equateStrings(cond, PacketError.RECIPIENT_UNAVAILABLE_CONDITION) && (type == PacketError.WAIT))
                codeVal = 404;
            else if (Utilities.equateStrings(cond, PacketError.REDIRECT_CONDITION) && (type == PacketError.MODIFY))
                codeVal = 302;
            else if (Utilities.equateStrings(cond, PacketError.REGISTRATION_REQUIRED_CONDITION) && (type == PacketError.AUTH))
                codeVal = 407;
            else if (Utilities.equateStrings(cond, PacketError.REMOTE_SERVER_NOT_FOUND_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 404;
            else if (Utilities.equateStrings(cond, PacketError.REMOTE_SERVER_TIMEOUT_CONDITION) && (type == PacketError.WAIT))
                codeVal = 504;
            else if (Utilities.equateStrings(cond, PacketError.RESOURCE_CONSTRAINT_CONDITION) && (type == PacketError.WAIT))
                codeVal = 500;
            else if (Utilities.equateStrings(cond, PacketError.SERVICE_UNAVAILABLE_CONDITION) && (type == PacketError.CANCEL))
                codeVal = 503;
            else if (Utilities.equateStrings(cond, PacketError.SUBSCRIPTION_REQUIRED_CONDITION) && (type == PacketError.AUTH))
                codeVal = 407;
            else if (Utilities.equateStrings(cond, PacketError.UNEXPECTED_REQUEST_CONDITION) && (type == PacketError.WAIT))
                codeVal = 400;
            else
                codeVal = 500;
            
            err.setAttributeValue("code", Integer.toString(codeVal));
        }
    }
    protected void configureIETF(PacketError err) {
        PacketError.Type    type = err.getType();
        String              cond = err.getDefinedCondition();
        String              msg = err.normalizeTrimText();

        if (!Utilities.isValidString(cond)) {
            try {
                switch (Integer.parseInt(err.getAttributeValue("code"))) {
                    case 302:
                        cond = PacketError.REDIRECT_CONDITION;
                        type = PacketError.MODIFY;
                        break;
                    case 400:
                        cond = PacketError.BAD_REQUEST_CONDITION;
                        type = PacketError.MODIFY;
                        break;
                    case 401:
                        cond = PacketError.NOT_AUTHORIZED_CONDITION;
                        type = PacketError.AUTH;
                        break;
                    case 402:
                        cond = PacketError.PAYMENT_REQUIRED_CONDITION;
                        type = PacketError.AUTH;
                        break;
                    case 403:
                        cond = PacketError.FORBIDDEN_CONDITION;
                        type = PacketError.AUTH;
                        break;
                    case 404:
                        cond = PacketError.ITEM_NOT_FOUND_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    case 405:
                        cond = PacketError.NOT_ALLOWED_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    case 406:
                        cond = PacketError.NOT_ACCEPTABLE_CONDITION;
                        type = PacketError.MODIFY;
                        break;
                    case 407:
                        cond = PacketError.REGISTRATION_REQUIRED_CONDITION;
                        type = PacketError.AUTH;
                        break;
                    case 408:
                        cond = PacketError.REMOTE_SERVER_TIMEOUT_CONDITION;
                        type = PacketError.WAIT;
                        break;
                    case 409:
                        cond = PacketError.CONFLICT_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    case 500:
                        cond = PacketError.INTERNAL_SERVER_ERROR_CONDITION;
                        type = PacketError.WAIT;
                        break;
                    case 501:
                        cond = PacketError.FEATURE_NOT_IMPLEMENTED_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    case 502:
                        cond = PacketError.SERVICE_UNAVAILABLE_CONDITION;
                        type = PacketError.WAIT;
                    case 503:
                        cond = PacketError.SERVICE_UNAVAILABLE_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    case 504:
                        cond = PacketError.REMOTE_SERVER_TIMEOUT_CONDITION;
                        type = PacketError.WAIT;
                        break;
                    case 510:
                        cond = PacketError.SERVICE_UNAVAILABLE_CONDITION;
                        type = PacketError.CANCEL;
                        break;
                    default:
                        cond = PacketError.UNDEFINED_CONDITION;
                        type = PacketError.CANCEL;
                }
                
                err.setType(type);
                err.setDefinedCondition(cond);
            } catch (NumberFormatException nfe) {
            }
        }
    }
}