/*
 * PacketErrorNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jaxen.JaxenException;
import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.ErrorCondition;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamObjectVisitor;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Implementation of the <code>PacketError</code> interface.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class PacketErrorNode extends ElementNode implements PacketError {
    //"Constants"
    public static final NSI    NAME             = new NSI("error", null);

    public static final NSI     ELEMNAME_TEXT   = new NSI("text", CONDITION_NAMESPACE);
    private static final NSI    ATTRNAME_TYPE   = new NSI("type", null);
    private static final NSI    ATTRNAME_CODE   = new NSI("code", null);
    
    private static final Map    DEFINED_CONDITIONS;
    
    static {
        Map     temp = new java.util.TreeMap();
        NSI     name;
        
        name = new NSI(BAD_REQUEST_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(CONFLICT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(FEATURE_NOT_IMPLEMENTED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(FORBIDDEN_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(GONE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(INTERNAL_SERVER_ERROR_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(ITEM_NOT_FOUND_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(JID_MALFORMED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(NOT_ACCEPTABLE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(NOT_ALLOWED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(NOT_AUTHORIZED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(PAYMENT_REQUIRED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(RECIPIENT_UNAVAILABLE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(REDIRECT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(REGISTRATION_REQUIRED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(REMOTE_SERVER_NOT_FOUND_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(REMOTE_SERVER_TIMEOUT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(RESOURCE_CONSTRAINT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(SERVICE_UNAVAILABLE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(SUBSCRIPTION_REQUIRED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNDEFINED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNEXPECTED_REQUEST_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        
        DEFINED_CONDITIONS = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Fields
    private XPath   _AppCondPath;

    //Constructors
    public PacketErrorNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    /**
     *
     * @deprecated As of JSO-0.12, this constructor is removed with no
     * replacement.
     */
    public PacketErrorNode(StreamDataFactory sdf, int code, String text) {
        this(sdf);
        setCode(code);
        add(new TextNode(this, sdf, text));
    }
    public PacketErrorNode(StreamElement parent, PacketErrorNode cp) {
        super(parent, cp);
    }
    
    //Methods
    protected XPath getApplicationConditionXPath() {
        if (_AppCondPath == null) {
            try {
                _AppCondPath = getDataFactory().createXPath("*[namespace-uri()!='" + getNamespaceURI() + "' and namespace-uri()!='" + CONDITION_NAMESPACE + "']");
            } catch (SAXPathException saxpe) {
            }
        }
        
         return _AppCondPath;
    }
    
    public StreamBuilder createBuilder() {
        return new ElementBuilder(this) {
            protected StreamElement createElementNode(StreamParseState s) {
                StreamElement       ctx = s.getContext();
                NSI.Comparator      cmpr = NSI.NS_LOOSE_COMPARATOR;
                NSI                 name = s.getName();
                String              uri = name.getNamespaceURI();
                String              ln = name.getLocalName();
                StreamDataFactory   sdf = ctx.getDataFactory();
                StreamElement       elem;
                
                if (Utilities.equateStrings(PacketError.CONDITION_NAMESPACE, uri) && Utilities.equateStrings("text", ln)) {
                    elem = sdf.createElementNode(name);
                } else {
                    elem = sdf.createElementNode(name, ErrorCondition.class);
                }
                
                return elem;
            }
        };
    }
    
    public List listConditions() {
        return listElements(ErrorCondition.class);
    }
    
    public String getDefinedCondition() {
        StreamElement   condition = getFirstElement(null, CONDITION_NAMESPACE, ErrorCondition.class);
        
        return ((condition != null) ? condition.getLocalName() : "");
    }
    public void setDefinedCondition(String name) {
        Iterator        itr = listElements(null, CONDITION_NAMESPACE, ErrorCondition.class).iterator();
        NSI             nsi;
        
        //Remove any existing
        while (itr.hasNext()) {
            ((StreamElement)itr.next()).detach();
        }

        if (Utilities.isValidString(name)) {
            if ((nsi = (NSI)DEFINED_CONDITIONS.get(name)) == null)
                throw new IllegalArgumentException("\"" + nsi + "\" is not a defined condition");
            addElement(nsi, ErrorCondition.class);
        }
    }
    
    public NSI getApplicationCondition() {
        StreamElement   elem;
        NSI             name = null;
        
        try {
            elem = (StreamElement)getApplicationConditionXPath().selectSingleNode(this);
            
            if (elem != null)
                name = elem.getNSI();
        } catch (JaxenException je) {
        }
        
        return name;
    }
    public void setApplicationCondition(NSI name) throws IllegalArgumentException {
        Iterator    itr;
        
        //Validate input
        if (name != null) {
            if (!Utilities.isValidString(name.getLocalName()))
                throw new IllegalArgumentException("localname cannot be null or \"\"");
            if (Utilities.equateStrings(name.getNamespaceURI(), getNSI().getNamespaceURI()))
                throw new IllegalArgumentException("namespace URI cannot be in stanza namespace");
            if (Utilities.equateStrings(name.getNamespaceURI(), CONDITION_NAMESPACE))
                throw new IllegalArgumentException("namespace URI cannot be in stanza-conditions namespace");
        }
        
        //Out the old...
        try {
            itr = getApplicationConditionXPath().selectNodes(this).iterator();
            
            while (itr.hasNext())
                ((StreamElement)itr.next()).detach();
        } catch (JaxenException je) {
        }
        
        //...And in with the new!
        if (name != null) {
            addElement(name, ErrorCondition.class);
        }
    }
    
    public PacketError.Type getType() {
        Object              val = getAttributeObject(ATTRNAME_TYPE);
        PacketError.Type    type = null;
        
        if          (val instanceof PacketError.Type) {
            type = (PacketError.Type)val;
        } else if   (val != null) {
            String  temp = val.toString();
            
            if      (Utilities.equateStrings(CANCEL.toString(), temp))
                type = CANCEL;
            else if (Utilities.equateStrings(CONTINUE.toString(), temp))
                type = CONTINUE;
            else if (Utilities.equateStrings(MODIFY.toString(), temp))
                type = MODIFY;
            else if (Utilities.equateStrings(AUTH.toString(), temp))
                type = AUTH;
            else if (Utilities.equateStrings(WAIT.toString(), temp))
                type = WAIT;
            
            if (type != null)
                setAttributeObject(ATTRNAME_TYPE, type);
        }
        
        return type;
    }
    public void setType(PacketError.Type type) {
        setAttributeObject(ATTRNAME_TYPE, type);
    }
    
    /** <p>
     * Retrieves the numeric code for this <code>PacketError</code>.  This
     * code is roughly equivalent to the HTTP error of the same meaning.</p>
     *
     * The value returned by this method is never less than 0.</p>
     *
     * @return The numeric code for this <code>PacketError</code>.
     */
    public int getCode() {
        Object      val = getAttributeObject(ATTRNAME_CODE);
        int         code;
        //Integer     val = (Integer)getAttributeObject(ATTRIBUTE_CODE);
        if (val instanceof Integer)
            code = ((Integer)val).intValue();
        else
            code = 0;
        
        return code;
    }
    /**
     * <p>
     * Sets the numeric code for this <Code>PacketError</code>.</p>
     *
     * <p>
     * The value of <code>code</code> cannot be less than 0, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param code The new code for this <code>PacketError</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     */
    public void setCode(int code) {
        if (code < 0)
            throw new IllegalArgumentException("Code cannot be less than 0");
        setAttributeObject(ATTRNAME_CODE, new Integer(code));
    }
    
    
    
    public String getText() {
        StreamElement   text = getFirstElement(ELEMNAME_TEXT);
        
        return ((text != null) ? text.normalizeTrimText() : "");
    }
    public void setText(String txt) {
        Iterator    itr = listElements(ELEMNAME_TEXT).iterator();
        
        while (itr.hasNext())
            ((StreamElement)itr.next()).detach();
        
        if (txt != null)
            addElement(ELEMNAME_TEXT).addText(txt);
    }
    
    /**
     * <p>
     * Compares this <code>PacketError</code> the given object for equality.
     * This comparison can take place against <code>PacketError</code>, Number,
     * or a String.</p>
     *
     * <p>
     * This method performs the same comparison as <code>{@link
     * #compareTo(Object)}</code>.  If that method returns 0, this method
     * returns <code>true</code>, otherwise <code>false</code> is returned.
     * Any exceptions that may be thrown by <code>{@link
     * #compareTo(Object)}</code> result in this method returning
     * <code>false</code>.</p>
     *
     * @param cmp The object to test.
     * @return <code>true</code> if the two are equal.
     */    
    public boolean equals(Object obj) {
        boolean     result;

        try {
            result = (compareTo(obj) == 0);
        } catch (Exception e) {
            result = false;
        }

        return result;
    }
    /**
     * <p>
     * Compares this <tt>PacketError</tt> instance against the object. This
     * comparison can be made against <tt>PacketError</tt>, <tt>Number</tt>,
     * or a String.  The behavior of this method depends upon the value of
     * <code>obj</code>:
     * <ul>
     *   <li>If <code>cmp</code> is an instance of <code>PacketError</code>,
     * then this comparison first tests the error codes, then the texts.  The
     * error code test results in the "difference" between this
     * <code>PacketError</code> and <code>obj</code>.  The text test uses the
     * String method <code>compareToIgnoreCase()</code>.  In any case, this
     * method returns a value less than 0 if this <code>PacketError</code> comes
     * before <code>obj</code>, greater than 0 if it comes after, and 0 if they
     * are "positionally" equal.</li>
     *   <li>If <code>cmp</code> is an instance of <code>Number</code>, then the
     * result is the difference between this <code>PacketError</code> object's
     * <code>{@link #getCode}</code> and <code>obj</code>.</li>
     *   <li>If <code>cmp</code> is any other type of object, an integer
     * "conversion" of its <code>toString()</code> results are used.  If this
     * conversion fails, a <code>ClassCastException</code> is thrown.</p>
     *
     * <p>
     * If the value of </code>cmp</code> is <code>null</code>, a
     * <code>NullPointerException</code> is thrown.</p>
     *
     * @param cmp The object to test.
     * @return The results of the object comparison.
     */
    public int compareTo(Object obj) {
        PacketError         cmp = (PacketError)obj;
        PacketError.Type    type1, type2;
        String              temp1, temp2;
        int                 result = 0;

        if (obj == null)
            throw new NullPointerException();
        type1 = getType();
        type2 = cmp.getType();

        //Compare types
        if (type1 == null)
            type1 = PacketError.CANCEL;
        if (type2 == null)
            type2 = PacketError.CANCEL;
        result = type1.compareTo(type2);

        if (result == 0) {
            //Compare conditions
            temp1 = getDefinedCondition();
            temp2 = cmp.getDefinedCondition();

            if (!Utilities.isValidString(temp1))
                temp1 = UNDEFINED_CONDITION;
            if (!Utilities.isValidString(temp2))
                temp2 = UNDEFINED_CONDITION;
            result = temp1.compareTo(temp2);
        }
        
        return result;
    }
    
    /** <p>
     * Creates an "almost-exact" duplicate of this <code>StreamNode</code>,
     * using the given parent <code>StreamElement</code>.  By "almost-exact",
     * all information for this <code>StreamNode</code>, and its contained
     * objects (obtained via <code>{@link #listNodes}</code>) are duplicated.
     * The parent for this <code>StreamNode</code> is one specified by
     * <code>p</code>, rather than thatreferenced by this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case no
     * parent is attached to the newly created <code>StreamNode</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamNode</code>.
     */
    public StreamObject copy(StreamElement p) {
        return new PacketErrorNode(p, this);
    }
}