/*
 * DOMReaderImpl.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.DOMImporter;
import org.jabberstudio.jso.io.StreamBuilder;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class DOMReader implements DOMImporter {
    //Fields
    private StreamElement   _Ctx;
    
    private StreamElement   _Output;
    private List            _BuildStack = new java.util.LinkedList();
    
    //Constructors
    public DOMReader() {
    }
    public DOMReader(StreamElement ctx) throws IllegalArgumentException {
        configure(ctx);
    }
    
    //Methods
    public StreamElement getContext() {
        return _Ctx;
    }
    private void setContext(StreamElement ctx) throws IllegalArgumentException {
        if (ctx == null)
            throw new IllegalArgumentException("context StreamElement cannot be null");
        _Ctx = ctx;
    }
    
    public StreamElement read(Element in) {
        setOutput(null);
        readElement(null, null, in);
        
        return getOutput();
    }

    private void readElement(DOMParseState parent, DOMParseState current, Element in) {
        StreamBuilder           b;
        StreamBuilder.Status    status;
        Iterator                itr;

        //Prepare "current" ParseState
        if (current == null)
            current = DOMParseState.createForElement(getContext(), parent, in);
        
        //Setup builder and children
        b = setupBuilder(current, topBuilder());
        itr = current.getNodes().iterator();
        
        //Process Element Start
        current.setType(DOMParseState.ELEMENT_START);
        if (b.processElementStart(current) == StreamBuilder.EXTENDED_DATA) {
            pushBuilder(b.buildExtendedData());
            readElement(parent, current, in);
            return;
        }
        
        //Process Element Children
        while (itr.hasNext()) {
            Node    child = (Node)itr.next();
            
            if (child instanceof Element) {
                readElement(current, null, (Element)child);
            } else if (child instanceof Text) {
                readText(current, (Text)child);
            }
        }
        
        //Process Element End
        current.setType(DOMParseState.ELEMENT_END);
        if ((b != null) && (b.processElementEnd(current) == StreamBuilder.COMPLETE)) {
            StreamNode      node = b.getNode();
            
            popBuilder();
            b = topBuilder();
            if (b == null)
                setOutput((StreamElement)node);
            else
                b.addExtendedData(node);
        }
    }
    private void readText(DOMParseState parent, Text in) {
        DOMParseState           current = DOMParseState.createForText(getContext(), parent, in);
        StreamBuilder           b = topBuilder();
        
        b.processText(current);
    }
    
    private StreamElement getOutput() {
        return _Output;
    }
    private void setOutput(StreamElement out) {
        _Output = out;
    }
    
    private StreamBuilder setupBuilder(DOMParseState current, StreamBuilder b) {
        if (b == null) {
            StreamElement       ctx = getContext();
            StreamDataFactory   sdf;
            NSI                 name = current.getName();

            //Check 
            if (name.equals(ctx.getNSI())) {
                b = ctx.createBuilder();
            } else {
                b = ctx.getDataFactory().createElementBuilder(name, null);
            }
            
            pushBuilder(b);
        }
        
        return b;
    }
    
    private StreamBuilder topBuilder() {
        return (StreamBuilder)(_BuildStack.isEmpty() ? null : _BuildStack.get(0));
    }
    private void pushBuilder(StreamBuilder b) {
        _BuildStack.add(0, b);
    }
    private StreamBuilder popBuilder() {
        return (StreamBuilder)(_BuildStack.isEmpty() ? null : _BuildStack.remove(0));
    }
    
    public void configure(StreamElement ctx) throws IllegalArgumentException, IllegalStateException, RuntimeException {
        if (getContext() != null)
            throw new IllegalStateException("This importer is already configured");
        setContext(ctx);
    }
    
    public StreamElement execute(Object in) throws RuntimeException {
        return read((Element)in);
    }
}