/*
 * ItemNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.disco;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.disco.DiscoItem;
import org.jabberstudio.jso.x.disco.DiscoUtilities;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class ItemNode extends ElementNode implements DiscoItem {
    //"Constants"
    public static final NSI     ATTRNAME_ACTION = new NSI("action", null);
    public static final NSI     ATTRNAME_JID    = new NSI("jid", null);
    
    //Constructors
    public ItemNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected ItemNode(StreamElement parent, ItemNode base) {
        super(parent, base);
    }
    
    //Methods    
    public JID getJID() {
        JID     jid = null;
        Object  val = getAttributeObject(ATTRNAME_JID);
        
        if          (val instanceof JID) {
            jid = (JID)val;
        } else if   (val != null) {
            try {
                jid = JID.valueOf(val);
            } catch (IllegalArgumentException iae) {
            }
        }
        
        return jid;
    }
    public void setJID(JID jid) {
        setAttributeObject(ATTRNAME_JID, jid);
    }
    
    public String getNode() {
        String  node = getAttributeValue(DiscoUtilities.ATTRNAME_NODE);
        
        return (Utilities.isValidString(node) ? node : "");
    }
    public void setNode(String node) {
        setAttributeValue(DiscoUtilities.ATTRNAME_NODE, Utilities.isValidString(node) ? node : null);
    }
    
    public String getName() {
        String  name = getAttributeValue(DiscoUtilities.ATTRNAME_NAME);
        
        return (Utilities.isValidString(name) ? name : "");
    }
    public void setName(String name) {
        setAttributeValue(DiscoUtilities.ATTRNAME_NAME, name);
    }
    
    public Action getAction() {
        Action  act = null;
        Object  val = getAttributeObject(ATTRNAME_ACTION);

        if          (val instanceof Action) {
            act = (Action)val;
        } else if   (val != null) {
            String  name = val.toString();
            
            if          (Utilities.equateStrings(REMOVE.toString(), name)) {
                act = REMOVE;
            } else if   (Utilities.equateStrings(UPDATE.toString(), name)) {
                act = UPDATE;
            }
            
            if (act != null)
                setAttributeObject(ATTRNAME_ACTION, act);
        }
        
        return act;
    }
    public void setAction(Action act) {
        setAttributeObject(ATTRNAME_ACTION, act);
    }

    public StreamObject copy(StreamElement parent) {
        return new ItemNode(parent, this);
    }
}