/*
 * ItemsNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.disco;


import java.util.Iterator;
import java.util.List;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.disco.DiscoItem;
import org.jabberstudio.jso.x.disco.DiscoItemsQuery;
import org.jabberstudio.jso.x.disco.DiscoUtilities;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class ItemsNode extends ExtensionNode implements DiscoItemsQuery {
    //Constructors
    public ItemsNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected ItemsNode(StreamElement parent, ItemsNode base) {
        super(parent, base);
    }
    
    //Methods
    public String getNode() {
        String  node = getAttributeValue(DiscoUtilities.ATTRNAME_NODE);
        
        return (Utilities.isValidString(node) ? node : "");
    }
    public void setNode(String node) {
        setAttributeValue(DiscoUtilities.ATTRNAME_NODE, node);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new ItemsNode(parent, this);
    }
    
    public List listItems() {
        return listElements(DiscoItem.class);
    }
    public DiscoItem getItem(org.jabberstudio.jso.JID jid) throws IllegalArgumentException {
        return getItem(jid, null);
    }
    public DiscoItem getItem(JID jid, String node) throws IllegalArgumentException {
        DiscoItem       item = null, temp;
        Iterator        itr;
        
        if (jid == null)
            throw new IllegalArgumentException("JID cannot be null");
        if (node == null)
            node = "";
        
        itr = listItems().iterator();
        while ((item == null) && itr.hasNext()) {
            temp = (DiscoItem)itr.next();
            
            if (JID.STRICT_COMPARATOR.compare(temp.getJID(), jid) != 0)
                continue;
            if (!Utilities.equateStrings(temp.getNode(), node))
                continue;
            item = temp;
        }
        
        return item;
    }
    public DiscoItem addItem(JID jid) throws IllegalArgumentException {
        return addItem(jid, null, null);
    }
    public DiscoItem addItem(JID jid, String node) throws IllegalArgumentException {
        return addItem(jid, node, null);
    }
    public DiscoItem addItem(JID jid, String node, String name) throws IllegalArgumentException {
        DiscoItem   item = getItem(jid, node);
        
        if (item == null) {
            ItemNode    temp = (ItemNode)addElement(ItemNode.NAME, ItemNode.class);
            
            temp.setJID(jid);
            temp.setNode(node);
            temp.setName(name);
            
            item = temp;
        }
        
        return item;
    }
    public void removeItem(JID jid) throws IllegalArgumentException {
        removeItem(jid, null);
    }
    public void removeItem(JID jid, String node) throws IllegalArgumentException {
        DiscoItem   item = getItem(jid, node);
        
        if (item != null)
            item.detach();
    }
    public void clearItems() {
        clearElements(DiscoItem.class);
    }
}