/*
 * InfoExtensionFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Date;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.AbstractElementFactory;

/**
 *
 * @author  linuxwolf
 */
public class InfoExtensionFactory extends AbstractElementFactory {
    //"Constants"
    public static final String  OS_STRING = MessageFormat.format("{0}/{1} ({2})", new Object[] {
        System.getProperty("os.name"),
        System.getProperty("os.arch"),
        System.getProperty("os.version")
    });
    
    public InfoExtensionFactory() {
        super(NSI.STRICT_COMPARATOR);
    }
    
    protected void registerSupportedElements() throws IllegalArgumentException {
        putSupportedElement(new DelayExtensionNode(null));
        putSupportedElement(new ExpireExtensionNode(null));
        putSupportedElement(new LastQueryNode(null));
        putSupportedElement(OOBExtensionNode.createOutOfBandIQ(null));
        putSupportedElement(OOBExtensionNode.createOutOfBandX(null));
        putSupportedElement(new TimeQueryNode(null));
        putSupportedElement(new VersionQueryNode(null));
    }
    
    public StreamElement createElementNode(NSI name, Class iface, StreamDataFactory sdf) {
        StreamElement   elem = super.createElementNode(name, iface, sdf);

        name = elem.getNSI();
        if          (NSI.STRICT_COMPARATOR.compare(name, DelayExtensionNode.NAME) == 0) {
            DelayExtensionNode  delay = (DelayExtensionNode)elem;
            Date                now = new Date();
            
            delay.setStamp(now);
        } else if   (NSI.STRICT_COMPARATOR.compare(name, TimeQueryNode.NAME) == 0) {
            TimeQueryNode   time = (TimeQueryNode)elem;
            DateFormat      fmt = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG);
            Date            now = new Date();
            
            time.setTime(now);
            time.setTimezone(java.util.TimeZone.getDefault().getDisplayName());
            time.setDisplay(fmt.format(now));
        } else if   (NSI.STRICT_COMPARATOR.compare(name, VersionQueryNode.NAME) == 0) {
            VersionQueryNode    ver = (VersionQueryNode)elem;
            
            ver.setName("JSO-based Application");
            ver.setOS(OS_STRING);
            ver.setVersion(Utilities.VERSION);
        }
        
        return elem;
    }
}