/*
 * StreamNodeFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamText;

/**
 * <p>
 * Interface for creating "primitive" stream information nodes.  The nodes
 * created via this interface should be modifiable.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 * @deprecated As of JSO-0.12, this interface is removed. All functionality
 * provided by this interface is now located in {@link
 * org.jabberstudio.jso.StreamDataFactory}.
 */
public interface StreamNodeFactory {
    //Methods
    /**
     * <p>
     * Creates a new <code>NSI</code> object with the given local name and
     * namespace URI.  The factory may return a cached value, rather than
     * create a new object.</p>
     *
     * <p>
     * The value of <code>name</code> can be <code>null</code>.  Its value
     * cannot be "", or an <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>uri</code> can be <code>null</code> or "".   The
     * "non-namspace" is considered "".</p>
     *
     * @param name The local name for the <code>NSI</code>.
     * @param uri The namespace URI for the <code>NSI</code>.
     * @return The "new" <code>NSI</code> object.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public NSI createNSI(String name, String uri) throws IllegalArgumentException;

    /**
     * <p>
     * Creates a new <code>StreamText</code> object with the given text value.
     * This new <code>StreamText</code> does not have any parent reference.</p>
     *
     * <p>
     * The value of <code>txt</code> can be <code>null</code> or "".  If it is
     * <code>null</code>, it is reconsidered to be "".</p>
     *
     * @param txt  The text value string for the <code>StreamText</code>.
     * @return the new <code>StreamText</code>.
     * @since   JSO-0.2
     */
    public StreamText createText(String txt);

    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given local name, and
     * no namespace (or the "system default" namespace).  If a "non-namespace"
     * is desired, called <code>{@link #createElement(String, String)}</code>
     * with <code>null</code> for the last argument.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The local name for this <code>StreamElement</code>.
     * @return the new <code>StreamElement</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public StreamElement createElement(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given local name and
     * namespace.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>uri</code> can be <code>null</code> or "".</p>
     *
     * @param name The local name for this <code>StreamElement</code>.
     * @param uri The namespace URI for this <code>StreamElement</code>.
     * @return The new <code>StreamElement</code>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.2
     */
    public StreamElement createElement(String name, String uri) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given name.  The name
     * is specified as an <code>NSI</code>, which contains the local name and
     * the governing namespace URI.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.  Also, if the value
     * returned by <code>{@link NSI#getLocalName}</code> is <code>null</code>
     * or "", this method throws an <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name for this <code>StreamElement</code>.
     * @return The new <code>StreamElement</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public StreamElement createElement(NSI name) throws IllegalArgumentException;
}