/*
 * DiscoIdentity.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.disco;


import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing a Service Discovery information identity.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
 */
public interface DiscoIdentity extends StreamElement {
    //"Constants"
    /**
     * The qualified name for this <tt>DiscoIdentity</tt>
     * ({http://jabber.org/protocol/disco#info}identity).
     *
     * @since   JSO-0.12
     */
    public static final NSI NAME = new NSI("identity", DiscoInfoQuery.NAMESPACE);

    //Methods
    /**
     * <p>
     * Retrieves the category for this <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The category for this <tt>DiscoIdentity</tt>.
     * @since   JSO-0.12
     */
    public String getCategory();
    /**
     * <p>
     * Retrieves the type for this <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The type for this <tt>DiscoIdentity</tt>.
     * @since   JSO-0.12
     */
    public String getType();

    /**
     * <p>
     * Retrieves the name for this <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The name for this <tt>DiscoIdentity</tt>.
     * @since   JSO-0.12
     */
    public String getName();
    /**
     * <p>
     * Sets the name for this <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed..</p>
     *
     * @param name The new name.
     * @since   JSO-0.12
     */
    public void setName(String name);
}